from pathlib import Path

import numpy as np
from PIL import Image

# Load project structure
PROJECT_DIR = Path(__file__).resolve(strict=True).parent.parent
SRC_DIR = PROJECT_DIR / "src"

# Create a local directory if it doesn't exist
LOCAL_DIR = PROJECT_DIR / "local"
LOCAL_DIR.mkdir(exist_ok=True)

# Create a local data directory if it doesn't exist
DATA_DIR = LOCAL_DIR / "data"
DATA_DIR.mkdir(exist_ok=True)

# Color maps for symbols according to arc-agi css
COLORS = {
    0: "#000000",  # black
    1: "#0074D9",  # blue
    2: "#FF4136",  # red
    3: "#2ECC40",  # green
    4: "#FFDC00",  # yellow
    5: "#AAAAAA",  # grey
    6: "#F012BE",  # fuchsia
    7: "#FF851B",  # orange
    8: "#7FDBFF",  # teal
    9: "#870C25",  # brown
}


# ==================== Utility functions ====================

# TODO: This is fully generated by copilot and I have no idea how it works
# I probably need to understand how it works or write tests to ensure it works as expected


def render(grid: np.ndarray, img_width: int = 200, img_height: int = 200):
    """Render the grid as an image"""
    rows, cols = grid.shape

    # Border size should be half a percent of the image width
    border_size = 1

    # Compute scale factor for the grid
    scale_width = max(1, img_width // max(1, cols))
    scale_height = max(1, img_height // max(1, rows))
    scale = min(scale_width, scale_height)

    # Convert color codes from hex to RGB
    color_rgb = np.array(
        [
            tuple(int(COLORS[i][k : k + 2], 16) for k in (1, 3, 5))
            for i in range(len(COLORS))
        ],
        dtype=np.uint8,
    )

    alpha = np.full((rows, cols, 1), 255, dtype=np.uint8)
    colored_grid = np.concatenate([color_rgb[grid], alpha], axis=-1)

    scaled_grid = np.kron(colored_grid, np.ones((scale, scale, 1), np.uint8))

    final_height = rows * scale + (rows - 1) * border_size
    final_width = cols * scale + (cols - 1) * border_size
    canvas = np.full((final_height, final_width, 4), (80, 80, 80, 255), dtype=np.uint8)

    row_idx = np.arange(rows * scale)
    col_idx = np.arange(cols * scale)
    row_idx += (row_idx // scale) * border_size
    col_idx += (col_idx // scale) * border_size
    canvas[row_idx[:, None], col_idx[None, :]] = scaled_grid

    return Image.fromarray(canvas, "RGBA")
